var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { commonsArrayShuffle, commonsNumberAverage } from 'tscommons-es-core';
export function commonsAiCrossValidator(constructorCallback, data, k = 4, comparator, options = {}, trainingOptions = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        const clone = data.slice();
        commonsArrayShuffle(clone);
        const size = Math.floor(clone.length / k);
        if (size < 4)
            throw new Error(`Training set size is too small for ${clone.length} k folds of ${k}`);
        const blocks = [];
        for (let i = 0; i < k; i++) {
            const dclone = clone.slice();
            const testingData = dclone.splice(i * size, size);
            const trainingData = dclone;
            blocks.push({
                trainingData: trainingData,
                testingData: testingData
            });
        }
        const trainers = [];
        for (const block of blocks) {
            const trainer = constructorCallback(options);
            trainer.train(block.trainingData, trainingOptions);
            let testResult;
            if (comparator) {
                const errors = [];
                for (const test of block.testingData) {
                    const output = trainer.run(test.input);
                    const compared = yield comparator(test.input, output);
                    errors.push(compared);
                }
                testResult = {
                    error: commonsNumberAverage(errors)
                };
            }
            else {
                testResult = trainer.test(block.testingData);
            }
            trainers.push({
                trainer: trainer,
                testError: testResult.error
            });
        }
        let best = trainers.shift();
        for (const trainer of trainers) {
            if (trainer.testError < best.testError)
                best = trainer;
        }
        return best.trainer;
    });
}
//# sourceMappingURL=commons-ai-cross-validator.js.map